// ChoiceScript (c) Dan Fabulich
// smPlugin.js - CJW @ www.choiceofgames.com/forum
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this file, to utilize it without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
//  * The 'usage' of this file is kept within compliance of the 'ChoiceScript License'.
//    You can obtain a copy of the license at http://www.choiceofgames.com/LICENSE-1.0.txt
//  
//  * The copyright, attributions and permission notices must be retained 
//    within all copies of the code (modified or otherwise).

//  *Unless required by applicable law or agreed to in writing, the
//    licensor provides the work on an "AS IS" BASIS, WITHOUT WARRANTIES OR
//    CONDITIONS OF ANY KIND, either express or implied, including, without
//    limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT,
//    MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely
//    responsible for determining the appropriateness of using or
//    redistributing the works and assume any risks associated with your
//    exercise of permissions under this license.
//
//  * In no event and under no legal theory, whether in tort (including
//	  negligence), contract, or otherwise, unless required by applicable law
//    (such as deliberate and grossly negligent acts) or agreed to in
//    writing, shall the licensor be liable to you for damages, including
//    any direct, indirect, special, incidental, or consequential damages of
//    any character arising as a result of this license or out of the use or
//    inability to use the works (including but not limited to damages for
//    loss of goodwill, work stoppage, computer failure or malfunction, or
//    any and all other commercial damages or losses), even if the licensor
//    has been advised of the possibility of such damages.

(function(){
 "use strict";



var saveMod	=	{
					game_id: "",
					slotCount: 3,
					justLoaded: false
				};

//Validate New Commands
Scene.validCommands.sm_load = 1;
Scene.validCommands.sm_save = 1;
Scene.validCommands.sm_delete = 1;
Scene.validCommands.sm_init = 1;
Scene.validCommands.whatever = 1;

Scene.prototype.sm_init = function sm_init(data) {
try {
    data = data || "";
    var args = data.split(" | ");
        if (args[0]) {
            saveMod.game_id = args[0].toLowerCase();        
        }
        else { 
            throw new Error("SaveMod: Non existent game_id argument for *init_savemod!");
        }
        if (args[1]) {
            saveMod.slotCount = Number(args[1]);
        }
        else { 
            throw new Error("SaveMod: Non existent slotCount argument for *init_savemod!");
        }
    
    for (var i=0;i<=saveMod.slotCount;i++) {
        if (localStorage[saveMod.game_id + "_savemod_name_" + i]) {
            stats["savemod_slot_" + i] = localStorage[saveMod.game_id + "_savemod_name_" + i];
        }
        else {
            stats["savemod_slot_" + i] = "No Data";
        }
    }
	}
catch(err) {
	alertify.error("SaveMod Error: localStorage is not supported!" );
		for (var i=0;i<=saveMod.slotCount;i++) {
		stats["savemod_slot_" + i] = "Saving is not supported";
		}
	}
}


Scene.prototype.sm_load = function sm_load(data) {
try {
if (saveMod.justLoaded) { return; } //If just loaded, do nothing.
    if (!saveMod.game_id) {
    throw new Error("SaveMod: Unable to *load before *init_savemod");
    }

    data = data || "";
    var args = data.split(" | ");
    if (args.length != 2) { throw new Error("SaveMod: Expected two arguments for *load, got " + args.length); }
    
    var saveSlot = args[0];
    if ((isNaN(saveSlot))) {throw new Error("SaveMod: '" + saveSlot + "' is not a valid save slot!"); }
    var doPrompt = args[1];
        doPrompt = str_to_boolean(doPrompt);


    if (!localStorage[saveMod.game_id + "_savemod_data_" + args[0]]) {
         alertify.error("There is no saved data!" ); return; 
    }
        
    if (doPrompt) {
        var x = confirm( "Are you sure you wish to load this game?\nYour current progress will be lost.");
            if (x) {
                    window.stats.scene.restore_localStorage(saveSlot);
            }
            else {
                return;
            }
    }
    else {
        window.stats.scene.restore_localStorage(saveSlot);
    }
	}
catch(err) {
	alertify.error("SaveMod Error: localStorage is not supported!" ); return; 
	}	
}

Scene.prototype.sm_save = function sm_save(data) {
try {
if (saveMod.justLoaded) { saveMod.justLoaded = false; return; } //If just loaded, don't try and save.
    if (!saveMod.game_id) {
    throw new Error("SaveMod: Unable to *save before *init_savemod");
    }

    data = data || "";
    var args = data.split(" | ");
     if (args.length != 2 && args.length != 3) { throw new Error("SaveMod: Expected two or three arguments for *save, got " + args.length); return; }
    
    var num = Number(args[0]);
    var isNum = isNaN(num);
    if (!isNum) {  
        if (num > saveMod.slotCount) {
            throw new Error("SaveMod: Specified save slot is out of bounds!");
        }
    }
    else {
        throw new Error("SaveMod: Specified save slot must be a number!"); 
    }
    
    var saveSlot = args[0];
    if ((isNaN(saveSlot))) {throw new Error("SaveMod: '" + saveSlot + "' is not a valid save slot!"); }
    var doPrompt = args[1];
        doPrompt = str_to_boolean(doPrompt);
      
    if (args[2]) {      
        var saveName = args[2];
        }
	else {
		var saveName = getSaveDate();
	}
        
    if (doPrompt) {
        var x = confirm( "Are you sure you wish to save your game?\nThe data in the selected slot will be overwritten.");
            if (x) {
                var x = prompt( "What would you like to name your save?",saveName);
                    if (x) {
                            localStorage[saveMod.game_id + "_savemod_data_" + saveSlot] = saveMod.c_password;
                            localStorage[saveMod.game_id + "_savemod_name_" + saveSlot] = x;
                            stats["savemod_slot_" + saveSlot] = x;
							alertify.success(x + " saved successfully!" );
							return;
							}
                        else { 
                            alertify.error("No save name specified!" ); return; 
							}
							}
			else { return; }
					}
    else if (!saveName) {
		var saveDateName = getSaveDate();
		
        localStorage[saveMod.game_id + "_savemod_data_" + saveSlot] = saveMod.c_password;
        localStorage[saveMod.game_id + "_savemod_name_" + saveSlot] = saveDateName;
        stats["savemod_slot_" + saveSlot] = saveDateName; //Save Game Autosave
    }
    else {
        localStorage[saveMod.game_id + "_savemod_data_" + saveSlot] = saveMod.c_password;
        localStorage[saveMod.game_id + "_savemod_name_" + saveSlot] = saveName;
        stats["savemod_slot_" + saveSlot] = saveName;
    }
	}
catch(err) {
	if (!localStorage) { alertify.error("SaveMod Error: localStorage is not supported!" ); return; }
	else { alert("Reportable Error: " + err); return;}
	}
}

Scene.prototype.sm_delete = function sm_delete(data) {
try {
    if (!saveMod.game_id) {
    throw new Error("SaveMod: Unable to *delete before *init_savemod");
    }
    data = data || "";
    if (data === "") {
            throw new Error("SaveMod: No target slot given for *delete!");
    }
    var num = Number(data);
    var isNum = isNaN(num);
    if (!isNum) {  
        if (num > saveMod.slotCount) {
            throw new Error("SaveMod: Specified save slot is out of bounds!"); 
        }
    }
    else {
        throw new Error("SaveMod: Specified save slot must be a number!"); 
    }
    if (localStorage[saveMod.game_id + "_savemod_name_" + data] || localStorage[saveMod.game_id + "_savemod_data_" + data]) {
        var x = confirm( "Are you sure you wish to delete your saved game?\nThis action cannot be undone.");
            if (x) {
                var saveID = saveMod.game_id + "_savemod_name_" + data;
                var saveData = saveMod.game_id + "_savemod_data_" + data;
                localStorage.removeItem(saveID);
                localStorage.removeItem(saveData);
                stats["savemod_slot_" + data] = "No Data";
                alertify.success("Deletion succesful!" );
            }
            else {
                return;
            }
    }
    else { 
        alertify.error("There is nothing to delete!" ); return;
    }
	}
catch(err) {
	alertify.error("SaveMod Error: localStorage is not supported!" ); return; 
	}
}

Scene.prototype.restore_localStorage = function restore_localStorage(saveSlot) {
try {
    if (!saveMod.game_id) {
    throw new Error("SaveMod: Unable to *delete before *init_savemod");
    }
    saveSlot = saveSlot || "";
    if (saveSlot === "") {
            throw new Error("SaveMod: No target slot given for *load!");
    }
    var num = Number(saveSlot);
    var isNum = isNaN(num);
    if (!isNum) {  
        if (num > saveMod.slotCount) {
            throw new Error("SaveMod: Specified load slot is out of bounds!"); 
        }
    }
    else {
        throw new Error("SaveMod: Specified load slot must be a number!"); 
    }
    
    saveMod.justLoaded = true; //Prevent load/save loops
    var alreadyFinished = this.finished;
    var self = this;
    var unrestorableScenes = this.parseRestoreGame(alreadyFinished);

    var password = localStorage[saveMod.game_id + "_savemod_data_" + saveSlot];
        password = password.replace(/\s/g, "");
        password = password.replace(/^.*BEGINPASSWORD-----/, "");
        var token = this.deobfuscatePassword(password);
        token = token.replace(/^[^\{]*/, "");
        token = token.replace(/[^\}]*$/, "");
        try {
          var state = jsonParse(token);
        } catch (e) {
          var supportEmail = "support-unknown@choiceofgames.com";
          try {
            supportEmail=document.getElementById("supportEmail").getAttribute("href");
            supportEmail=supportEmail.replace(/\+/g,"%2B");
            supportEmail=supportEmail.replace(/mailto:/, "");
          } catch (e) {
            supportEmail = "support-unknown@choiceofgames.com";
          }
          alert("Sorry, that password was invalid. Please contact " + supportEmail + " for assistance. Be sure to include your password in the email.");
          return;
        }
        
        var sceneName = null;
        if (state.stats && state.stats.sceneName) sceneName = (""+state.stats.sceneName).toLowerCase();
        
        var unrestorable = unrestorableScenes[sceneName];
        if (unrestorable) {
          alert(unrestorable);
          self.finished = false;
          self.resetPage();
          return;
        }
        saveCookie(function() {
          clearScreen(function() {
            // we're going to pretend not to be user restored, so we get reprompted to save
            restoreGame(state, null, /*userRestored*/false);
          })
        }, "", state.stats, state.temps, state.lineNum, state.indent, this.debugMode, this.nav);
 alertify.success("Load succesful!" );
	}
catch(err) {
	alertify.error("SaveMod Error: localStorage is not supported!" ); return; 
	}
}

//Overwrites a native cs interpreter function
// reset the page and invoke code after clearing the screen
Scene.prototype.resetPage = function resetPage() {
    if (typeof(Storage)!=="undefined") {
        var scene = window.stats.scene;

        var password = computeCookie(scene.stats, scene.temps, scene.lineNum, scene.indent);
        password = scene.obfuscate(password);
        password = "----- BEGIN PASSWORD -----\n" + password + "\n----- END PASSWORD -----";
        saveMod.c_password = password; //Stores password but doesn't "save it".
    } 
    var self = this;
    this.save(function() {
      self.prevLineEmpty = true;
      self.screenEmpty = true;
      clearScreen(function() {self.execute();});
    }, "");
}

function str_to_boolean(string) {
    string = string.toLowerCase();
    if (string == "true" || string == "false") {
        if (string == "true") {
            return true;
        }
        else {
            return false;
        }
    }
    else {
        throw new Error("SaveMod: Second argument for *load must be either true or false!"); 
    }
}

function getSaveDate() {
	    var date = new Date();
        var year = date.getFullYear();
        var day = date.getDate();
        var month = date.getMonth();
        var hours = date.getHours();
        var mins = date.getMinutes();
        date = day + "/" + month + "/" + year;
        var time = hours + ":" + mins;
		var saveName = "Autosave on the " + date + " at " + time;
		return saveName;
}

Scene.prototype.whatever = function whatever() {
 
 var testObject = { 'one': 1, 'two': 2, 'three': 3 };
 
var seen = []
// Put the object into storage
localStorage.setItem('statss', JSON.stringify(stats, function(key, val) {
   if (typeof val == "object") {
        if (seen.indexOf(val) >= 0) {
            return
        seen.push(val);
    }
	}
    return val;
}));

// Retrieve the object from storage
var retrievedObject = localStorage.getItem('statss');

console.log('retrievedObject: ', JSON.parse(retrievedObject));

}

})();
